@@ -1,6 +1,6 @@
 /*
  * ProFTPD - mod_proxy database implementation
- * Copyright (c) 2015-2020 TJ Saunders
+ * Copyright (c) 2015-2021 TJ Saunders
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -140,11 +140,11 @@ static int db_trace2(unsigned int trace_type, void *user_data, void *ptr,
     case SQLITE_TRACE_PROFILE: {
       sqlite3_stmt *pstmt;
       int64_t ns = 0;
-      const char *expanded_sql = NULL;
+      char *expanded_sql = NULL, *orig_sql = NULL;
 
       pstmt = ptr;
       ns = *((int64_t *) ptr_data);
-      expanded_sql = sqlite3_expanded_sql(pstmt);
+      orig_sql = expanded_sql = sqlite3_expanded_sql(pstmt);
 
       /* There are some SQL statements whose values we do NOT want to log.
        * Thus we have a hacky way to look for them.  Sigh.
@@ -165,15 +165,16 @@ static int db_trace2(unsigned int trace_type, void *user_data, void *ptr,
           expanded_sql, (unsigned long) ns);
       }
 
+      sqlite3_free(orig_sql);
       break;
     }
 
     case SQLITE_TRACE_ROW: {
       sqlite3_stmt *pstmt;
-      const char *expanded_sql = NULL;
+      char *expanded_sql = NULL, *orig_sql = NULL;
 
       pstmt = ptr;
-      expanded_sql = sqlite3_expanded_sql(pstmt);
+      orig_sql = expanded_sql = sqlite3_expanded_sql(pstmt);
 
       /* There are some SQL statements whose values we do NOT want to log.
        * Thus we have a hacky way to look for them.  Sigh.
@@ -193,6 +194,7 @@ static int db_trace2(unsigned int trace_type, void *user_data, void *ptr,
           schema_name, expanded_sql);
       }
 
+      sqlite3_free(orig_sql);
       break;
     }
 
@@ -306,11 +308,9 @@ int proxy_db_exec_stmt(pool *p, struct proxy_dbh *dbh, const char *stmt,
     return -1;
   }
 
-  if (ptr != NULL) {
-    sqlite3_free(ptr);
-  }
-
   current_schema = NULL;
+  sqlite3_free(ptr);
+
   pr_trace_msg(trace_channel, 13, "successfully executed '%s'", stmt);
   return 0;
 }
@@ -330,6 +330,13 @@ int proxy_db_prepare_stmt(pool *p, struct proxy_dbh *dbh, const char *stmt) {
 
   pstmt = (sqlite3_stmt *) pr_table_get(dbh->prepared_stmts, stmt, NULL);
   if (pstmt != NULL) {
+    res = sqlite3_clear_bindings(pstmt);
+    if (res != SQLITE_OK) {
+      pr_trace_msg(trace_channel, 3,
+        "error clearing bindings from prepared statement '%s': %s", stmt,
+        sqlite3_errmsg(dbh->db));
+    }
+
     res = sqlite3_reset(pstmt);
     if (res != SQLITE_OK) {
       pr_trace_msg(trace_channel, 3,
@@ -368,7 +375,7 @@ int proxy_db_prepare_stmt(pool *p, struct proxy_dbh *dbh, const char *stmt) {
 }
 
 int proxy_db_bind_stmt(pool *p, struct proxy_dbh *dbh, const char *stmt,
-    int idx, int type, void *data) {
+    int idx, int type, void *data, int datalen) {
   sqlite3_stmt *pstmt;
   int res;
  
@@ -448,7 +455,7 @@ int proxy_db_bind_stmt(pool *p, struct proxy_dbh *dbh, const char *stmt,
       }
 
       text = (const char *) data;
-      res = sqlite3_bind_text(pstmt, idx, text, -1, NULL);
+      res = sqlite3_bind_text(pstmt, idx, text, datalen, NULL);
       if (res != SQLITE_OK) {
         pr_trace_msg(trace_channel, 4,
           "error binding parameter %d of '%s' to TEXT '%s': %s", idx, stmt,
@@ -459,6 +466,23 @@ int proxy_db_bind_stmt(pool *p, struct proxy_dbh *dbh, const char *stmt,
       break;
     }
 
+    case PROXY_DB_BIND_TYPE_BLOB: {
+      if (data == NULL) {
+        errno = EINVAL;
+        return -1;
+      }
+
+      res = sqlite3_bind_blob(pstmt, idx, data, datalen, NULL);
+      if (res != SQLITE_OK) {
+        pr_trace_msg(trace_channel, 4,
+          "error binding parameter %d of '%s' to BLOB (%d bytes): %s", idx,
+          stmt, datalen, sqlite3_errmsg(dbh->db));
+        errno = EPERM;
+        return -1;
+      }
+      break;
+    }
+
     case PROXY_DB_BIND_TYPE_NULL:
       res = sqlite3_bind_null(pstmt, idx);
       if (res != SQLITE_OK) {
@@ -559,7 +583,7 @@ array_header *proxy_db_exec_prepared_stmt(pool *p, struct proxy_dbh *dbh,
       const char *errmsg;
 
       errmsg = sqlite3_errmsg(dbh->db);
-      if (errstr) {
+      if (errstr != NULL) {
         *errstr = pstrdup(p, errmsg);
       }
       pr_trace_msg(trace_channel, 2,
@@ -594,17 +618,36 @@ array_header *proxy_db_exec_prepared_stmt(pool *p, struct proxy_dbh *dbh,
 
     for (i = 0; i < ncols; i++) {
       char *val = NULL;
+      int col_type;
 
       pr_signals_handle();
 
-      /* By using sqlite3_column_text, SQLite will coerce the column value
-       * into a string.
-       */
-      val = pstrdup(p, (const char *) sqlite3_column_text(pstmt, i));
+      col_type = sqlite3_column_type(pstmt, i);
 
-      pr_trace_msg(trace_channel, 17,
-        "column %s [%u]: %s", sqlite3_column_name(pstmt, i), i, val);
-      *((char **) push_array(results)) = val;
+      if (col_type != SQLITE_BLOB) {
+        /* By using sqlite3_column_text, SQLite will coerce the column value
+         * into a string.
+         */
+        val = pstrdup(p, (const char *) sqlite3_column_text(pstmt, i));
+        pr_trace_msg(trace_channel, 17,
+          "column %s [%u]: %s", sqlite3_column_name(pstmt, i), i, val);
+
+        *((char **) push_array(results)) = val;
+
+      } else {
+        int bloblen;
+        char bloblen_text[64];
+
+        bloblen = sqlite3_column_bytes(pstmt, i);
+        val = palloc(p, bloblen);
+        memcpy(val, sqlite3_column_blob(pstmt, i), bloblen);
+        *((char **) push_array(results)) = val;
+
+        /* For BLOBs, we need to provide the length as well. */
+        memset(&bloblen_text, '\0', sizeof(bloblen_text));
+        pr_snprintf(bloblen_text, sizeof(bloblen_text)-1, "%d", bloblen);
+        *((char **) push_array(results)) = pstrdup(p, bloblen_text);
+      }
     }
 
     res = sqlite3_step(pstmt);
@@ -669,9 +712,10 @@ struct proxy_dbh *proxy_db_open(pool *p, const char *table_path,
     return NULL;
   }
 
-  if (pr_trace_get_level(trace_channel) >= PROXY_DB_SQLITE_TRACE_LEVEL) {
-    sqlite3_busy_handler(db, db_busy, (void *) schema_name);
+  /* Make sure we configure a busy handler. */
+  sqlite3_busy_handler(db, db_busy, (void *) schema_name);
 
+  if (pr_trace_get_level(trace_channel) >= PROXY_DB_SQLITE_TRACE_LEVEL) {
 #if defined(HAVE_SQLITE3_TRACE_V2)
     sqlite3_trace_v2(db, SQLITE_TRACE_STMT|SQLITE_TRACE_PROFILE|SQLITE_TRACE_ROW|SQLITE_TRACE_CLOSE,
       db_trace2, (void *) schema_name);
@@ -734,7 +778,7 @@ static int get_schema_version(pool *p, struct proxy_dbh *dbh,
   }
 
   res = proxy_db_bind_stmt(p, dbh, stmt, 1, PROXY_DB_BIND_TYPE_TEXT,
-    (void *) schema_name);
+    (void *) schema_name, -1);
   if (res < 0) {
     return -1;
   }
@@ -800,13 +844,13 @@ static int set_schema_version(pool *p, struct proxy_dbh *dbh,
   }
 
   res = proxy_db_bind_stmt(p, dbh, stmt, 1, PROXY_DB_BIND_TYPE_TEXT,
-    (void *) schema_name);
+    (void *) schema_name, -1);
   if (res < 0) {
     return -1;
   }
 
   res = proxy_db_bind_stmt(p, dbh, stmt, 2, PROXY_DB_BIND_TYPE_INT,
-    (void *) &schema_version);
+    (void *) &schema_version, 0);
   if (res < 0) {
     return -1;
   }
