@@ -1,6 +1,6 @@
 /*
  * ProFTPD - mod_proxy forward proxy implementation
- * Copyright (c) 2012-2020 TJ Saunders
+ * Copyright (c) 2012-2021 TJ Saunders
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -198,6 +198,10 @@ static int forward_connect(pool *p, struct proxy_session *proxy_sess,
   dst_addr = proxy_sess->dst_addr;
   other_addrs = proxy_sess->other_addrs;
 
+  if (proxy_tls_using_tls() == PROXY_TLS_ENGINE_MATCH_CLIENT) {
+    proxy_tls_match_client_tls();
+  }
+
   /* If the destination port is 990, assume implicit FTPS. */
   if (ntohs(pr_netaddr_get_port(dst_addr)) == PROXY_TLS_IMPLICIT_FTPS_PORT) {
     pr_trace_msg(trace_channel, 9, "%s#%u requesting, using implicit FTPS",
@@ -513,7 +517,7 @@ static int forward_handle_user_passthru(cmd_rec *cmd,
         pr_netaddr_get_ipstr(session.c->local_addr),
         ntohs(pr_netaddr_get_port(session.c->local_addr)));
       pr_response_send(R_530, _("Unable to connect to %s: %s"),
-        proxy_conn_get_host(pconn), strerror(EPERM));
+        proxy_conn_get_hostport(pconn), strerror(EPERM));
       return 1;
     }
 
@@ -551,7 +555,7 @@ static int forward_handle_user_passthru(cmd_rec *cmd,
 
       } else {
         resp->msg = pstrcat(cmd->tmp_pool, "Unable to connect to ",
-          proxy_conn_get_host(proxy_sess->dst_pconn), ": ",
+          proxy_conn_get_hostport(proxy_sess->dst_pconn), ": ",
           strerror(xerrno), NULL);
         resp_nlines = 1;
       }
