@@ -1,6 +1,6 @@
 /*
  * ProFTPD - mod_proxy testsuite
- * Copyright (c) 2013-2020 TJ Saunders <tj@castaglia.org>
+ * Copyright (c) 2013-2021 TJ Saunders <tj@castaglia.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -521,8 +521,9 @@ START_TEST (reverse_handle_user_pass_random_test) {
   const struct proxy_conn *pconn;
   array_header *backends;
 
-  /* Skip this test on travis, for now.  It fails unexpectedly. */
-  if (getenv("TRAVIS") != NULL) {
+  /* Skip this test on CI builds, for now.  It fails unexpectedly. */
+  if (getenv("CI") != NULL ||
+      getenv("TRAVIS") != NULL) {
     return;
   }
 
@@ -546,8 +547,9 @@ START_TEST (reverse_handle_user_pass_roundrobin_test) {
   const struct proxy_conn *pconn;
   array_header *backends;
 
-  /* Skip this test on travis, for now.  It fails unexpectedly. */
-  if (getenv("TRAVIS") != NULL) {
+  /* Skip this test on CI builds, for now.  It fails unexpectedly. */
+  if (getenv("CI") != NULL ||
+      getenv("TRAVIS") != NULL) {
     return;
   }
 
@@ -571,8 +573,9 @@ START_TEST (reverse_handle_user_pass_leastconns_test) {
   const struct proxy_conn *pconn;
   array_header *backends;
 
-  /* Skip this test on travis, for now.  It fails unexpectedly. */
-  if (getenv("TRAVIS") != NULL) {
+  /* Skip this test on CI builds, for now.  It fails unexpectedly. */
+  if (getenv("CI") != NULL ||
+      getenv("TRAVIS") != NULL) {
     return;
   }
 
@@ -596,8 +599,9 @@ START_TEST (reverse_handle_user_pass_leastresponsetime_test) {
   const struct proxy_conn *pconn;
   array_header *backends;
 
-  /* Skip this test on travis, for now.  It fails unexpectedly. */
-  if (getenv("TRAVIS") != NULL) {
+  /* Skip this test on CI builds, for now.  It fails unexpectedly. */
+  if (getenv("CI") != NULL ||
+      getenv("TRAVIS") != NULL) {
     return;
   }
 
@@ -623,8 +627,9 @@ START_TEST (reverse_handle_user_pass_shuffle_test) {
   const struct proxy_conn *pconn;
   array_header *backends;
 
-  /* Skip this test on travis, for now.  It fails unexpectedly. */
-  if (getenv("TRAVIS") != NULL) {
+  /* Skip this test on CI builds, for now.  It fails unexpectedly. */
+  if (getenv("CI") != NULL ||
+      getenv("TRAVIS") != NULL) {
     return;
   }
 
@@ -648,8 +653,9 @@ START_TEST (reverse_handle_user_pass_peruser_test) {
   const struct proxy_conn *pconn;
   array_header *backends;
 
-  /* Skip this test on travis, for now.  It fails unexpectedly. */
-  if (getenv("TRAVIS") != NULL) {
+  /* Skip this test on CI builds, for now.  It fails unexpectedly. */
+  if (getenv("CI") != NULL ||
+      getenv("TRAVIS") != NULL) {
     return;
   }
 
@@ -673,8 +679,9 @@ START_TEST (reverse_handle_user_pass_pergroup_test) {
   const struct proxy_conn *pconn;
   array_header *backends;
 
-  /* Skip this test on travis, for now.  It fails unexpectedly. */
-  if (getenv("TRAVIS") != NULL) {
+  /* Skip this test on CI builds, for now.  It fails unexpectedly. */
+  if (getenv("CI") != NULL ||
+      getenv("TRAVIS") != NULL) {
     return;
   }
 
@@ -698,8 +705,9 @@ START_TEST (reverse_handle_user_pass_perhost_test) {
   const struct proxy_conn *pconn;
   array_header *backends;
 
-  /* Skip this test on travis, for now.  It fails unexpectedly. */
-  if (getenv("TRAVIS") != NULL) {
+  /* Skip this test on CI builds, for now.  It fails unexpectedly. */
+  if (getenv("CI") != NULL ||
+      getenv("TRAVIS") != NULL) {
     return;
   }
 
@@ -903,64 +911,64 @@ START_TEST (reverse_json_parse_uris_usable_test) {
 }
 END_TEST
 
-START_TEST (reverse_connect_get_policy_test) {
+START_TEST (reverse_connect_get_policy_id_test) {
   int res;
   const char *policy;
 
-  res = proxy_reverse_connect_get_policy(NULL);
+  res = proxy_reverse_connect_get_policy_id(NULL);
   fail_unless(res < 0, "Failed to handle null argument");
   fail_unless(errno == EINVAL, "Expected EINVAL (%d), got '%s' (%d)", EINVAL,
     strerror(errno), errno);
 
   policy = "foo";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res < 0, "Failed to handle unsupported policy '%s'", policy);
   fail_unless(errno == ENOENT, "Expected ENOENT (%d), got '%s' (%d)", ENOENT,
     strerror(errno), errno);
 
   policy = "random2";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res < 0, "Failed to handle unsupported policy '%s'", policy);
   fail_unless(errno == ENOENT, "Expected ENOENT (%d), got '%s' (%d)", ENOENT,
     strerror(errno), errno);
 
   policy = "random";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res == PROXY_REVERSE_CONNECT_POLICY_RANDOM,
     "Failed to handle supported policy '%s'", policy);
 
   policy = "roundrobin";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res == PROXY_REVERSE_CONNECT_POLICY_ROUND_ROBIN,
     "Failed to handle supported policy '%s'", policy);
 
   policy = "shuffle";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res == PROXY_REVERSE_CONNECT_POLICY_SHUFFLE,
     "Failed to handle supported policy '%s'", policy);
 
   policy = "leastconns";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res == PROXY_REVERSE_CONNECT_POLICY_LEAST_CONNS,
     "Failed to handle supported policy '%s'", policy);
 
   policy = "peruser";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res == PROXY_REVERSE_CONNECT_POLICY_PER_USER,
     "Failed to handle supported policy '%s'", policy);
 
   policy = "pergroup";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res == PROXY_REVERSE_CONNECT_POLICY_PER_GROUP,
     "Failed to handle supported policy '%s'", policy);
 
   policy = "perhost";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res == PROXY_REVERSE_CONNECT_POLICY_PER_HOST,
     "Failed to handle supported policy '%s'", policy);
 
   policy = "leastresponsetime";
-  res = proxy_reverse_connect_get_policy(policy);
+  res = proxy_reverse_connect_get_policy_id(policy);
   fail_unless(res == PROXY_REVERSE_CONNECT_POLICY_LEAST_RESPONSE_TIME,
     "Failed to handle supported policy '%s'", policy);
 }
@@ -1027,7 +1035,7 @@ Suite *tests_get_reverse_suite(void) {
   tcase_add_test(testcase, reverse_json_parse_uris_empty_test);
   tcase_add_test(testcase, reverse_json_parse_uris_malformed_test);
   tcase_add_test(testcase, reverse_json_parse_uris_usable_test);
-  tcase_add_test(testcase, reverse_connect_get_policy_test);
+  tcase_add_test(testcase, reverse_connect_get_policy_id_test);
   tcase_add_test(testcase, reverse_use_proxy_auth_test);
   tcase_add_test(testcase, reverse_have_authenticated_test);
 
